#ifndef SMARTSWITCH_H
#define SMARTSWITCH_H

#if defined(__cplusplus) || defined(c_plusplus)
extern "C"
{
#endif

#ifndef _KERNEL
#include <asterisk/format.h>
#include <asterisk/netsock2.h>
#endif

struct media_proxy_channel_stats
{
    uint32_t            pkt_sent;
    uint32_t            pkt_rcvd;
    time_t              ts;
};

#ifndef _KERNEL
enum AccountingDirection
{
    adOriginate,
    adAnswer,
};

struct common_stats
{
    struct ast_format format;
    struct ast_format internal_format;
    unsigned int rtp_pkt_count;
    unsigned int rtp_pkt_lost;
    double jitter;
    unsigned int rtcp_pkt_sent;
};

struct tx_stats
{
    struct common_stats common;
    double rtt;
};

struct rx_stats
{
    struct common_stats common;
};

struct rtp_stats
{
    struct tx_stats tx;
    struct rx_stats rx;
};

struct smartswitch_callbacks
{
    /* Media Proxy */
    void (*media_proxy_init)();
    void (*media_proxy_cleanup)();
    int (*media_proxy_start)();
    void (*media_proxy_stop)();
    int (*media_proxy_delete_channel)(const struct ast_sockaddr* local);
    int (*media_proxy_create_channel)(const struct ast_sockaddr* local);
    int (*media_proxy_create_bridge)(const struct ast_sockaddr* local1, const struct ast_sockaddr* local2);
    int (*media_proxy_delete_bridge)(const struct ast_sockaddr* local);
    int (*media_proxy_set_nat)(const struct ast_sockaddr* local, unsigned nat);
    int (*media_proxy_set_tos)(const struct ast_sockaddr* local, unsigned tos);
    int (*media_proxy_set_remote)(const struct ast_sockaddr* local, const struct ast_sockaddr* remote);
    int (*media_proxy_get_channel_stats)(const struct ast_sockaddr* s, struct media_proxy_channel_stats* stats);

    /* RTP Stats */
    void (*save_rtp_stats)(const char* channel_id, const struct rtp_stats* stats);
    void (*save_format)(const struct ast_channel* chan);

    /* RADIUS */
    void (*radius_acct_start)(struct ast_channel* chan, const enum AccountingDirection direction);
    void (*radius_acct_stop)(struct ast_channel* chan);

    int (*get_name)(struct ast_channel* chan, char* buf, int len);

    /* Personalisation */
    const char* (*switch_filename)(const struct ast_channel *chan, const char* filename, const char *preflang);
    int (*check_function)(const struct ast_channel *chan, const char* function);
};

extern struct smartswitch_callbacks* smartswitch;
#endif

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif

#endif
